* census division aggregation with single representative household
*
* used by build/aggregate_benchmark.gms



* ------------------------------------------------------------------------------
* set gams options
* ------------------------------------------------------------------------------

* allow for end of line comments after a "!"
$eolcom !

* allow embedded set notation
$onembedded



* ------------------------------------------------------------------------------
* define the aggregated regions
* ------------------------------------------------------------------------------

* define the aggregated regions (census divisions)
set
  r_a                   aggregated regions /
    nen                 ! new england
    mat                 ! middle atlantic
    enc                 ! east north central
    wnc                 ! west north central
    sat                 ! south atlantic
    esc                 ! east south central
    wsc                 ! west south central
    mnt                 ! mountain
    pac                 ! pacific
  /;

* aggregated region mapping
set
  r_map(r_a,r)          aggregated region mapping /
    nen.(               ! new england
      ct                ! connecticut
      me                ! maine
      ma                ! massachusetts
      nh                ! new hampshire
      ri                ! rhode island
      vt                ! vermont
    )
    mat.(               ! middle atlantic
      nj                ! new jersey
      ny                ! new york
      pa                ! pennsylvania
    )
    enc.(               ! east north central
      in                ! indiana
      il                ! illinois
      mi                ! michigan
      oh                ! ohio
      wi                ! wisconsin
    )
    wnc.(               ! west north central
      ia                ! iowa
      ks                ! kansas
      mn                ! minnesota
      mo                ! missouri
      ne                ! nebraska
      nd                ! north dakota
      sd                ! south dakota
    )
    sat.(               ! south atlantic
      dc                ! district of columbia
      de                ! delaware
      fl                ! florida
      ga                ! georgia
      md                ! maryland
      nc                ! north carolina
      sc                ! south carolina
      va                ! virginia
      wv                ! west virginia
    )
    esc.(               ! east south central
      al                ! alabama
      ky                ! kentucky
      ms                ! mississippi
      tn                ! tennessee
    )
    wsc.(               ! west south central
      ar                ! arkansas
      la                ! louisiana
      ok                ! oklahoma
      tx                ! texas
    )
    mnt.(               ! mountain
      az                ! arizona
      co                ! colorado
      id                ! idaho
      nm                ! new mexico
      mt                ! montana
      ut                ! utah
      nv                ! nevada
      wy                ! wyoming
    )
    pac.(               ! pacific
      ak                ! alaska
      ca                ! california
      hi                ! hawaii
      or                ! oregon
      wa                ! washington
    )
  /;



* ------------------------------------------------------------------------------
* define the aggregated sectors
* ------------------------------------------------------------------------------

* define the aggregated sectors
set
  s_a                   aggregated sectors        /
    agf	                ! agriculture, forestry, fishing and hunting
    cru	                ! crude oil
    col	                ! coal mining
    min	                ! metal ore and nonmetalic mineral mining
    ele	                ! electric power
    gas	                ! natural gas
    wsu	                ! water, sewage, and other utilities
    con	                ! construction
    fbm	                ! food and beverage manufacturing
    wpm	                ! wood product manufacturing
    ref	                ! petroleum refineries
    chm	                ! chemical manufacturing
    prm	                ! plastics and rubber products manufacturing
    cem	                ! cement manufacturing
    pmm	                ! primary metal manufacturing
    fmm	                ! fabricated metal product manufacturing
    cpu                 ! electronics and technology manufacturing
    tem	                ! transportation equipment manufacturing
    bom	                ! other manufacturing
    trn	                ! transportation
    ttn	                ! truck transportation
    srv	                ! services
    hlt	                ! healthcare services
  /;

* aggregated sector mapping
set
  s_map(s_a,s)          aggregated sector mapping /
    agf.(               ! agriculture, forestry, fishing and hunting
      1                 ! oilseed farming                                                                                  11111-2
      2                 ! grain farming                                                                                    11113-6, 11119
      3                 ! vegetable and melon farming                                                                      1112
      4                 ! fruit farming                                                                                    111331-2, 111331-4, 111336*, 111339
      5                 ! tree nut farming                                                                                 111335, 111336*
      6                 ! greenhouse, nursery, and floriculture production                                                 1114, 1125*
      7                 ! tobacco farming                                                                                  11191
      8                 ! cotton farming                                                                                   11192
      9                 ! sugarcane and sugar beet farming                                                                 11193, 111991
      10                ! all other crop farming                                                                           11194, 111992, 111998
      11                ! beef cattle ranching and farming, including feedlots and dual-purpose ranching and farming       11211, 11213
      12                ! dairy cattle and milk production                                                                 11212
      13                ! poultry and egg production                                                                       1123
      14                ! animal production, except cattle and poultry and eggs                                            1122, 1124, 1125*, 1129
      15                ! forestry, forest products, and timber tract production                                           1131-2
      16                ! commercial logging                                                                               1133
      17                ! commercial fishing                                                                               1141
      18                ! commercial hunting and trapping                                                                  1142
      19                ! support activities for agriculture and forestry                                                  115
    )
    cru.(               ! crude oil
      20                ! extraction of natural gas and crude petroleum                                                    211111
    )
    col.(               ! coal mining
      22                ! coal mining                                                                                      2121
    )
    min.(               ! metal ore and nonmetalic mineral mining
      23                ! iron ore mining                                                                                  212210
      24                ! gold ore mining                                                                                  212221
      25                ! silver ore mining                                                                                212222
      26                ! lead and zinc ore mining                                                                         212231
      27                ! copper ore mining                                                                                212234
      28                ! uranium-radium-vanadium ore mining                                                               212291
      29                ! other metal ore mining                                                                           212299
      30                ! stone mining and quarrying                                                                       21231
      31                ! sand and gravel mining                                                                           212321-4
      32                ! other clay, ceramic, refractory minerals mining                                                  212325
      33                ! potash, soda, and borate mineral mining                                                          212391
      34                ! phosphate rock mining                                                                            212392
      35                ! other chemical and fertilizer mineral mining                                                     212393
      36                ! other nonmetallic minerals                                                                       212399
      39                ! metal mining services                                                                            213114
      40                ! other nonmetallic minerals services                                                              213113, 213115
    )
    ele.(               ! electric power
      41                ! electric power generation - hydroelectric                                                        221111
      42                ! electric power generation - fossil fuel                                                          221112
      43                ! electric power generation - nuclear                                                              221113
      44                ! electric power generation - solar                                                                221114
      45                ! electric power generation - wind                                                                 221115
      46                ! electric power generation - geothermal                                                           221116
      47                ! electric power generation - biomass                                                              221117
      48                ! electric power generation - all other                                                            221118
      49                ! electric power transmission and distribution
      519               ! federal electric utilities
      522               ! state government electric utilities
      525               ! local government electric utilities                                                              22112
    )
    gas.(               ! natural gas
      21                ! extraction of natural gas liquids                                                                211112
      50                ! natural gas distribution                                                                         2212
    )
    wsu.(               ! water, sewage, and other utilities
      51                ! water, sewage and other systems                                                                  2213
    )
    con.(               ! construction
      52                ! construction of new health care structures                                                       23*
      53                ! construction of new manufacturing structures                                                     23*
      54                ! construction of new power and communication structures                                           23*
      55                ! construction of new educational and vocational structures                                        23*
      56                ! construction of new highways and streets                                                         23*
      57                ! construction of new commercial structures, including farm structures                             23*
      58                ! construction of other new nonresidential structures                                              23*
      59                ! construction of new single-family residential structures                                         23*
      60                ! construction of new multifamily residential structures                                           23*
      61                ! construction of other new residential structures                                                 23*
      62                ! maintenance and repair construction of nonresidential structures                                 23*
      63                ! maintenance and repair construction of residential structures                                    23*
      64                ! maintenance and repair construction of highways, streets, bridges, and tunnels                   23*
    )
    fbm.(               ! food and beverage manufacturing
      65                ! dog and cat food manufacturing                                                                   311111
      66                ! other animal food manufacturing                                                                  311119
      67                ! flour milling                                                                                    311211
      68                ! rice milling                                                                                     311212
      69                ! malt manufacturing                                                                               311213
      70                ! wet corn milling                                                                                 311221
      71                ! soybean and other oilseed processing                                                             311224
      72                ! fats and oils refining and blending                                                              311225
      73                ! breakfast cereal manufacturing                                                                   31123
      74                ! beet sugar manufacturing                                                                         311313
      75                ! sugar cane mills and refining                                                                    311314
      76                ! nonchocolate confectionery manufacturing                                                         31134
      77                ! chocolate and confectionery manufacturing from cacao beans                                       311351
      78                ! confectionery manufacturing from purchased chocolate                                             311352
      79                ! frozen fruits, juices and vegetables manufacturing                                               311411
      80                ! frozen specialties manufacturing                                                                 311412
      81                ! canned fruits and vegetables manufacturing                                                       311421
      82                ! canned specialties                                                                               311422
      83                ! dehydrated food products manufacturing                                                           311423
      84                ! fluid milk manufacturing                                                                         311511
      85                ! creamery butter manufacturing                                                                    311512
      86                ! cheese manufacturing                                                                             311513
      87                ! dry, condensed, and evaporated dairy product manufacturing                                       311514
      88                ! ice cream and frozen dessert manufacturing                                                       31152
      89                ! animal, except poultry, slaughtering                                                             311611
      90                ! meat processed from carcasses                                                                    311612
      91                ! rendering and meat byproduct processing                                                          311613
      92                ! poultry processing                                                                               311615
      93                ! seafood product preparation and packaging                                                        3117
      94                ! bread and bakery product, except frozen, manufacturing                                           311811-2
      95                ! frozen cakes and other pastries manufacturing                                                    311813
      96                ! cookie and cracker manufacturing                                                                 311821
      97                ! dry pasta, mixes, and dough manufacturing                                                        311824
      98                ! tortilla manufacturing                                                                           31183
      99                ! roasted nuts and peanut butter manufacturing                                                     311911
      100               ! other snack food manufacturing                                                                   311919
      101               ! coffee and tea manufacturing                                                                     31192
      102               ! flavoring syrup and concentrate manufacturing                                                    31193
      103               ! mayonnaise, dressing, and sauce manufacturing                                                    311941
      104               ! spice and extract manufacturing                                                                  311942
      105               ! all other food manufacturing                                                                     31199
      106               ! bottled and canned soft drinks & water                                                           312111-2
      107               ! manufactured ice                                                                                 312113
      108               ! breweries                                                                                        31212
      109               ! wineries                                                                                         31213
      110               ! distilleries                                                                                     31214
      111               ! tobacco product manufacturing                                                                    3122
    )
    wpm.(               ! wood and paper product manufacturing
      134               ! sawmills                                                                                         321113
      135               ! wood preservation                                                                                321114
      136               ! veneer and plywood manufacturing                                                                 321211-2
      137               ! engineered wood member and truss manufacturing                                                   321213-4
      138               ! reconstituted wood product manufacturing                                                         321219
      139               ! wood windows and door manufacturing                                                              321911
      140               ! cut stock, resawing lumber, and planing                                                          321912
      141               ! other millwork, including flooring                                                               321918
      142               ! wood container and pallet manufacturing                                                          32192
      143               ! manufactured home (mobile home) manufacturing                                                    321991
      144               ! prefabricated wood building manufacturing                                                        321992
      145               ! all other miscellaneous wood product manufacturing                                               321999
      146               ! pulp mills                                                                                       32211
      147               ! paper mills                                                                                      32212
      148               ! paperboard mills                                                                                 32213
      149               ! paperboard container manufacturing                                                               32221
      150               ! paper bag and coated and treated paper manufacturing                                             32222
      151               ! stationery product manufacturing                                                                 32223
      152               ! sanitary paper product manufacturing                                                             322291
      153               ! all other converted paper product manufacturing                                                  322299
    )
    ref.(               ! petroleum refineries
      156               ! petroleum refineries                                                                             32411
    )
    chm.(               ! chemical manufacturing
      161               ! petrochemical manufacturing                                                                      32511
      162               ! industrial gas manufacturing                                                                     32512
      163               ! synthetic dye and pigment manufacturing                                                          32513
      164               ! other basic inorganic chemical manufacturing                                                     32518
      165               ! other basic organic chemical manufacturing                                                       32519
      166               ! plastics material and resin manufacturing                                                        325211
      167               ! synthetic rubber manufacturing                                                                   325212
      168               ! artificial and synthetic fibers and filaments manufacturing                                      32522
      169               ! nitrogenous fertilizer manufacturing                                                             325311
      170               ! phosphatic fertilizer manufacturing                                                              325312
      171               ! fertilizer mixing                                                                                325314
      172               ! pesticide and other agricultural chemical manufacturing                                          32532
      173               ! medicinal and botanical manufacturing                                                            325411
      174               ! pharmaceutical preparation manufacturing                                                         325412
      175               ! in-vitro diagnostic substance manufacturing                                                      325413
      176               ! biological product (except diagnostic) manufacturing                                             325414
      177               ! paint and coating manufacturing                                                                  32551
      178               ! adhesive manufacturing                                                                           32552
      179               ! soap and other detergent manufacturing                                                           325611
      180               ! polish and other sanitation good manufacturing                                                   325612
      181               ! surface active agent manufacturing                                                               325613
      182               ! toilet preparation manufacturing                                                                 32562
      183               ! printing ink manufacturing                                                                       32591
      184               ! explosives manufacturing                                                                         32592
      185               ! custom compounding of purchased resins                                                           325991
      186               ! photographic film and chemical manufacturing                                                     325992
      187               ! other miscellaneous chemical product manufacturing                                               325998
    )
    prm.(               ! plastics and rubber products manufacturing
      188               ! plastics packaging materials and unlaminated film and sheet manufacturing                        32611
      189               ! unlaminated plastics profile shape manufacturing                                                 326121
      190               ! plastics pipe and pipe fitting manufacturing                                                     326122
      191               ! laminated plastics plate, sheet (except packaging), and shape manufacturing                      326130
      192               ! polystyrene foam product manufacturing                                                           326140
      193               ! urethane and other foam product (except polystyrene) manufacturing                               326150
      194               ! plastics bottle manufacturing                                                                    326160
      195               ! other plastics product manufacturing                                                             326190
      196               ! tire manufacturing                                                                               326210
      197               ! rubber and plastics hoses and belting manufacturing                                              326220
      198               ! other rubber product manufacturing                                                               326290
    )
    cem.(               ! cement manufacturing
      205               ! cement manufacturing                                                                             32731
      206               ! ready-mix concrete manufacturing                                                                 32732
      207               ! concrete block and brick manufacturing                                                           327331
      208               ! concrete pipe manufacturing                                                                      327332
      209               ! other concrete product manufacturing                                                             32739
      210               ! lime manufacturing                                                                               32741
    )
    pmm.(               ! primary metal manufacturing
      217               ! iron and steel mills and ferroalloy manufacturing                                                33111
      218               ! iron, steel pipe and tube manufacturing from purchased steel                                     33121
      219               ! rolled steel shape manufacturing                                                                 331221
      220               ! steel wire drawing                                                                               331222
      221               ! alumina refining and primary aluminum production                                                 331313
      222               ! secondary smelting and alloying of aluminum                                                      331314
      223               ! aluminum sheet, plate, and foil manufacturing                                                    331315
      224               ! other aluminum rolling, drawing and extruding                                                    331318
      225               ! nonferrous metal (exc aluminum) smelting and refining                                            33141
      226               ! copper rolling, drawing, extruding and alloying                                                  33142
      227               ! nonferrous metal, except copper and aluminum, shaping                                            331491
      228               ! secondary processing of other nonferrous metals                                                  331492
      229               ! ferrous metal foundries                                                                          33151
      230               ! nonferrous metal foundries                                                                       33152
    )
    fmm.(               ! fabricated metal product manufacturing
      231               ! iron and steel forging                                                                           332111
      232               ! nonferrous forging                                                                               332112
      233               ! custom roll forming                                                                              332114
      234               ! crown and closure manufacturing and metal stamping                                               332117-9
      235               ! cutlery, utensil, pot, and pan manufacturing                                                     332215
      236               ! handtool manufacturing                                                                           332216
      237               ! prefabricated metal buildings and components manufacturing                                       332311
      238               ! fabricated structural metal manufacturing                                                        332312
      239               ! plate work manufacturing                                                                         332313
      240               ! metal window and door manufacturing                                                              332321
      241               ! sheet metal work manufacturing                                                                   332322
      242               ! ornamental and architectural metal work manufacturing                                            332323
      243               ! power boiler and heat exchanger manufacturing                                                    33241
      244               ! metal tank (heavy gauge) manufacturing                                                           33242
      245               ! metal cans manufacturing                                                                         332431
      246               ! metal barrels, drums and pails manufacturing                                                     332439
      247               ! hardware manufacturing                                                                           3325
      248               ! spring and wire product manufacturing                                                            3326
      249               ! machine shops                                                                                    33271
      250               ! turned product and screw, nut, and bolt manufacturing                                            33272
      251               ! metal heat treating                                                                              332811
      252               ! metal coating and nonprecious engraving                                                          332812
      253               ! electroplating, anodizing, and coloring metal                                                    332813
      254               ! valve and fittings, other than plumbing, manufacturing                                           332911, 332912, 332919
      255               ! plumbing fixture fitting and trim manufacturing                                                  332913
      256               ! ball and roller bearing manufacturing                                                            332991
      257               ! small arms ammunition manufacturing                                                              332992
      258               ! ammunition, except for small arms, manufacturing                                                 332993
      259               ! small arms, ordnance, and accessories manufacturing                                              332994
      260               ! fabricated pipe and pipe fitting manufacturing                                                   332996
      261               ! other fabricated metal manufacturing                                                             332999
    )
    cpu.(
      301               ! electronic computer manufacturing                                                                334111
      302               ! computer storage device manufacturing                                                            334112
      303               ! computer terminals and other computer peripheral equipment manufacturing                         334118
      304               ! telephone apparatus manufacturing                                                                33421
      305               ! broadcast and wireless communications equipment manufacturing                                    33422
      306               ! other communications equipment manufacturing                                                     33429
      307               ! audio and video equipment manufacturing                                                          3343
      308               ! bare printed circuit board manufacturing                                                         334412
      309               ! semiconductor and related device manufacturing                                                   334413
      310               ! capacitor, resistor, coil, transformer, and other inductor manufacturing                         334416
      311               ! electronic connector manufacturing                                                               334417
      312               ! printed circuit assembly (electronic assembly) manufacturing                                     334418
      313               ! other electronic component manufacturing                                                         334419
      314               ! electromedical and electrotherapeutic apparatus manufacturing                                    334510
      315               ! search, detection, and navigation instruments manufacturing                                      334511
      316               ! automatic environmental control manufacturing                                                    334512
      317               ! industrial process variable instruments manufacturing                                            334513
      318               ! totalizing fluid meter and counting device manufacturing                                         334514
      319               ! electricity and signal testing instruments manufacturing                                         334515
      320               ! analytical laboratory instrument manufacturing                                                   334516
      321               ! irradiation apparatus manufacturing                                                              334517
      322               ! watch, clock, and other measuring and controlling device manufacturing                           334519
      323               ! blank magnetic and optical recording media manufacturing                                         334613
      324               ! software and other prerecorded and record reproducing                                            334614
      325               ! electric lamp bulb and part manufacturing                                                        33511
      326               ! lighting fixture manufacturing                                                                   335120
      327               ! small electrical appliance manufacturing                                                         33521
      328               ! household cooking appliance manufacturing                                                        335221
      329               ! household refrigerator and home freezer manufacturing                                            335222
      330               ! household laundry equipment manufacturing                                                        335224
      331               ! other major household appliance manufacturing                                                    335228
      332               ! power, distribution, and specialty transformer manufacturing                                     335311
      333               ! motor and generator manufacturing                                                                335312
      334               ! switchgear and switchboard apparatus manufacturing                                               335313
      335               ! relay and industrial control manufacturing                                                       335314
      336               ! storage battery manufacturing                                                                    335911
      337               ! primary battery manufacturing                                                                    335912
      338               ! fiber optic cable manufacturing                                                                  335921
      339               ! other communication and energy wire manufacturing                                                335929
      340               ! wiring device manufacturing                                                                      33593
      341               ! carbon and graphite product manufacturing                                                        335991
      342               ! all other miscellaneous electrical equipment and component manufacturing                         335999
    )
    tem.(               ! transportation equipment manufacturing
      343               ! automobile manufacturing                                                                         336111
      344               ! light truck and utility vehicle manufacturing                                                    336112
      345               ! heavy duty truck manufacturing                                                                   33612
      346               ! motor vehicle body manufacturing                                                                 336211
      347               ! truck trailer manufacturing                                                                      336212
      348               ! motor home manufacturing                                                                         336213
      349               ! travel trailer and camper manufacturing                                                          336214
      350               ! motor vehicle gasoline engine and engine parts manufacturing                                     33631
      351               ! motor vehicle electrical and electronic equipment manufacturing                                  33632
      352               ! motor vehicle steering, suspension component (except spring), and brake systems manufacturing    33633-4
      353               ! motor vehicle transmission and power train parts manufacturing                                   33635
      354               ! motor vehicle seating and interior trim manufacturing                                            33636
      355               ! motor vehicle metal stamping                                                                     33637
      356               ! other motor vehicle parts manufacturing                                                          33639
      357               ! aircraft manufacturing                                                                           336411
      358               ! aircraft engine and engine parts manufacturing                                                   336412
      359               ! other aircraft parts and auxiliary equipment manufacturing                                       336413
      360               ! guided missile and space vehicle manufacturing                                                   336414
      361               ! propulsion units and parts for space vehicles and guided missiles manufacturing                  336415-9
      362               ! railroad rolling stock manufacturing                                                             3365
      363               ! ship building and repairing                                                                      336611
      364               ! boat building                                                                                    336612
      365               ! motorcycle, bicycle, and parts manufacturing                                                     336991
      366               ! military armored vehicle, tank, and tank component manufacturing                                 336992
      367               ! all other transportation equipment manufacturing                                                 336999
    )
    bom.(               ! balance of manufacturing
      37                ! drilling oil and gas wells                                                                       213111
      38                ! support activities for oil and gas operations                                                    213112
      112               ! fiber, yarn, and thread mills                                                                    3131
      113               ! broadwoven fabric mills                                                                          31321
      114               ! narrow fabric mills and schiffli machine embroidery                                              31322
      115               ! nonwoven fabric mills                                                                            31323
      116               ! knit fabric mills                                                                                31324
      117               ! textile and fabric finishing mills                                                               31331
      118               ! fabric coating mills                                                                             31332
      119               ! carpet and rug mills                                                                             31411
      120               ! curtain and linen mills                                                                          31412
      121               ! textile bag and canvas mills                                                                     31491
      122               ! rope, cordage, twine, tire cord and tire fabric mills                                            314994
      123               ! other textile product mills                                                                      314999
      124               ! hosiery and sock mills                                                                           31511
      125               ! other apparel knitting mills                                                                     31519
      126               ! cut and sew apparel contractors                                                                  31521
      127               ! men's and boys' cut and sew apparel manufacturing                                                31522
      128               ! women's and girls' cut and sew apparel manufacturing                                             31524
      129               ! other cut and sew apparel manufacturing                                                          31528
      130               ! apparel accessories and other apparel manufacturing                                              3159
      131               ! leather and hide tanning and finishing                                                           3161
      132               ! footwear manufacturing                                                                           3162
      133               ! other leather and allied product manufacturing                                                   3169
      154               ! printing                                                                                         32311
      155               ! support activities for printing                                                                  32312
      157               ! asphalt paving mixture and block manufacturing                                                   324121
      158               ! asphalt shingle and coating materials manufacturing                                              324122
      159               ! petroleum lubricating oil and grease manufacturing                                               324191
      160               ! all other petroleum and coal products manufacturing                                              324199
      199               ! pottery, ceramics, and plumbing fixture manufacturing                                            327110
      200               ! brick, tile, and other structural clay product manufacturing                                     327120
      201               ! flat glass manufacturing                                                                         327211
      202               ! other pressed and blown glass and glassware manufacturing                                        327212
      203               ! glass container manufacturing                                                                    327213
      204               ! glass product manufacturing made of purchased glass                                              327215
      211               ! gypsum product manufacturing                                                                     32742
      212               ! abrasive product manufacturing                                                                   32791
      213               ! cut stone and stone product manufacturing                                                        327991
      214               ! ground or treated mineral and earth manufacturing                                                327992
      215               ! mineral wool manufacturing                                                                       327993
      216               ! miscellaneous nonmetallic mineral products manufacturing                                         327999
      262               ! farm machinery and equipment manufacturing                                                       333111
      263               ! lawn and garden equipment manufacturing                                                          333112
      264               ! construction machinery manufacturing                                                             33312
      265               ! mining machinery and equipment manufacturing                                                     333131
      266               ! oil and gas field machinery and equipment manufacturing                                          333132
      267               ! food product machinery manufacturing                                                             333241
      268               ! semiconductor machinery manufacturing                                                            333242
      269               ! sawmill, woodworking, and paper machinery                                                        333243
      270               ! printing machinery and equipment manufacturing                                                   333244
      271               ! all other industrial machinery manufacturing                                                     333249
      272               ! optical instrument and lens manufacturing                                                        333314
      273               ! photographic and photocopying equipment manufacturing                                            333316
      274               ! other commercial service industry machinery manufacturing                                        333318
      275               ! air purification and ventilation equipment manufacturing                                         333413
      276               ! heating equipment (except warm air furnaces) manufacturing                                       333414
      277               ! air conditioning, refrigeration, and warm air heating equipment manufacturing                    333415
      278               ! industrial mold manufacturing                                                                    333511
      279               ! special tool, die, jig, and fixture manufacturing                                                333514
      280               ! cutting tool and machine tool accessory manufacturing                                            333515
      281               ! machine tool manufacturing                                                                       333517
      282               ! rolling mill and other metalworking machinery manufacturing                                      333519
      283               ! turbine and turbine generator set units manufacturing                                            333611
      284               ! speed changer, industrial high-speed drive, and gear manufacturing                               333612
      285               ! mechanical power transmission equipment manufacturing                                            333613
      286               ! other engine equipment manufacturing                                                             333618
      287               ! pump and pumping equipment manufacturing                                                         333911
      288               ! air and gas compressor manufacturing                                                             333912
      289               ! measuring and dispensing pump manufacturing                                                      333913
      290               ! elevator and moving stairway manufacturing                                                       333921
      291               ! conveyor and conveying equipment manufacturing                                                   333922
      292               ! overhead cranes, hoists, and monorail systems manufacturing                                      333923
      293               ! industrial truck, trailer, and stacker manufacturing                                             333924
      294               ! power-driven handtool manufacturing                                                              333991
      295               ! welding and soldering equipment manufacturing                                                    333992
      296               ! packaging machinery manufacturing                                                                333993
      297               ! industrial process furnace and oven manufacturing                                                333994
      298               ! fluid power cylinder and actuator manufacturing                                                  333995
      299               ! fluid power pump and motor manufacturing                                                         333996
      300               ! scales, balances, and miscellaneous general purpose machinery manufacturing                      333997-9
      368               ! wood kitchen cabinet and countertop manufacturing                                                33711
      369               ! upholstered household furniture manufacturing                                                    337121
      370               ! nonupholstered wood household furniture manufacturing                                            337122
      371               ! other household nonupholstered furniture manufacturing                                           337124-5
      372               ! institutional furniture manufacturing                                                            337127
      373               ! wood office furniture manufacturing                                                              337211
      374               ! custom architectural woodwork and millwork                                                       337212
      375               ! office furniture, except wood, manufacturing                                                     337214
      376               ! showcase, partition, shelving, and locker manufacturing                                          337215
      377               ! mattress manufacturing                                                                           33791
      378               ! blind and shade manufacturing                                                                    33792
      379               ! surgical and medical instrument manufacturing                                                    339112
      380               ! surgical appliance and supplies manufacturing                                                    339113
      381               ! dental equipment and supplies manufacturing                                                      339114
      382               ! ophthalmic goods manufacturing                                                                   339115
      383               ! dental laboratories                                                                              339116
      384               ! jewelry and silverware manufacturing                                                             33991
      385               ! sporting and athletic goods manufacturing                                                        33992
      386               ! doll, toy, and game manufacturing                                                                33993
      387               ! office supplies (except paper) manufacturing                                                     33994
      388               ! sign manufacturing                                                                               33995
      389               ! gasket, packing, and sealing device manufacturing                                                339991
      390               ! musical instrument manufacturing                                                                 339992
      391               ! fasteners, buttons, needles, and pins manufacturing                                              339993
      392               ! broom, brush, and mop manufacturing                                                              339994
      393               ! burial casket manufacturing                                                                      339995
      394               ! all other miscellaneous manufacturing                                                            339999
    )
    trn.(               ! transportation
      408               ! air transportation                                                                               481
      409               ! rail transportation                                                                              482
      410               ! water transportation                                                                             483
      412               ! transit and ground passenger transportation                                                      485
      413               ! pipeline transportation                                                                          486
      414               ! scenic and sightseeing transportation and support activities for transportation                  487, 488
      415               ! couriers and messengers                                                                          492
      416               ! warehousing and storage                                                                          493
      518               ! postal service                                                                                   491
      521               ! state government passenger transit
      524               ! local government passenger transit
    )
    ttn.(               ! truck transportation
      411               ! truck transportation                                                                             484
    )
    srv.(               ! services
      395               ! wholesale trade                                                                                  42
      396               ! retail - motor vehicle and parts dealers                                                         441
      397               ! retail - furniture and home furnishings stores                                                   442
      398               ! retail - electronics and appliance stores                                                        443
      399               ! retail - building material and garden equipment and supplies stores                              444
      400               ! retail - food and beverage stores                                                                445
      401               ! retail - health and personal care stores                                                         446
      402               ! retail - gasoline stores                                                                         447
      403               ! retail - clothing and clothing accessories stores                                                448
      404               ! retail - sporting goods, hobby, musical instrument and book stores                               451
      405               ! retail - general merchandise stores                                                              452
      406               ! retail - miscellaneious store retailers                                                          453
      407               ! retail - nonstore retailers                                                                      454
      417               ! newspaper publishers                                                                             51111
      418               ! periodical publishers                                                                            51112
      419               ! book publishers                                                                                  51113
      420               ! directory, mailing list, and other publishers                                                    51114, 511199
      421               ! greeting card publishing                                                                         511191
      422               ! software publishers                                                                              5112
      423               ! motion picture and video industries                                                              5121
      424               ! sound recording industries                                                                       5122
      425               ! radio and television broadcasting                                                                5151
      426               ! cable and other subscription programming                                                         5152
      427               ! wired telecommunications carriers                                                                51711
      428               ! wireless telecommunications carriers (except satellite)                                          51721
      429               ! satellite, telecommunications resellers, and all other telecommunications                        51741, 51791
      430               ! data processing, hosting, and related services                                                   5182
      431               ! news syndicates, libraries, archives and all other information services                          51911-2, 51919
      432               ! internet publishing and broadcasting and web search portals                                      51913
      433               ! monetary authorities and depository credit intermediation                                        5215221
      434               ! nondepository credit intermediation and related activities                                       5222-3
      435               ! securities and commodity contracts intermediation and brokerage                                  5231-2
      436               ! other financial investment activities                                                            5239
      437               ! insurance carriers                                                                               5241
      438               ! insurance agencies, brokerages, and related activities                                           5242
      439               ! funds, trusts, and other financial vehicles                                                      525
      440               ! real estate                                                                                      531
      441               ! owner-occupied dwellings                                                                         na
      442               ! automotive equipment rental and leasing                                                          5321
      443               ! general and consumer goods rental except video tapes and discs                                   532221-2, 53229, 5323
      444               ! video tape and disc rental                                                                       53223
      445               ! commercial and industrial machinery and equipment rental and leasing                             5324
      446               ! lessors of nonfinancial intangible assets                                                        533
      447               ! legal services                                                                                   5411
      448               ! accounting, tax preparation, bookkeeping, and payroll services                                   5412
      449               ! architectural, engineering, and related services                                                 5413
      450               ! specialized design services                                                                      5414
      451               ! custom computer programming services                                                             541511
      452               ! computer systems design services                                                                 541512
      453               ! other computer related services, including facilities management                                 54151a
      454               ! management consulting services                                                                   54161
      455               ! environmental and other technical consulting services                                            54162-9
      456               ! scientific research and development services                                                     5417
      457               ! advertising, public relations, and related services                                              5418
      458               ! photographic services                                                                            54192
      459               ! veterinary services                                                                              54194
      460               ! marketing research and all other miscellaneous professional, scientific, and technical services  54191, 54193, 54199
      461               ! management of companies and enterprises                                                          55
      462               ! office administrative services                                                                   5611
      463               ! facilities support services                                                                      5612
      464               ! employment services                                                                              5613
      465               ! business support services                                                                        5614
      466               ! travel arrangement and reservation services                                                      5615
      467               ! investigation and security services                                                              5616
      468               ! services to buildings                                                                            56171-2, 56174-9
      469               ! landscape and horticultural services                                                             56173
      470               ! other support services                                                                           5619
      471               ! waste management and remediation services                                                        562
      472               ! elementary and secondary schools                                                                 6111
      473               ! junior colleges, colleges, universities, and professional schools                                6112-3
      474               ! other educational services                                                                       6114-7
      483               ! nursing and community care facilities                                                            6231, 6233
      484               ! residential mental retardation, mental health, substance abuse and other facilities              6232, 6239
      485               ! individual and family services                                                                   6241
      486               ! community food, housing, and other relief services, including rehabilitation services            6242-3
      487               ! child day care services                                                                          6244
      488               ! performing arts companies                                                                        7111
      489               ! commercial sports except racing                                                                  711211, 711219
      490               ! racing and track operation                                                                       711212
      491               ! promoters of performing arts and sports and agents for public figures                            7113-4
      492               ! independent artists, writers, and performers                                                     7115
      493               ! museums, historical sites, zoos, and parks                                                       712
      494               ! amusement parks and arcades                                                                      7131
      495               ! gambling industries (except casino hotels)                                                       7132
      496               ! other amusement and recreation industries                                                        71391-3, 71399
      497               ! fitness and recreational sports centers                                                          71394
      498               ! bowling centers                                                                                  71395
      499               ! hotels and motels, including casino hotels                                                       72111-2
      500               ! other accommodations                                                                             72119, 7212-3
      501               ! full-service restaurants                                                                         722511
      502               ! limited-service restaurants                                                                      722513
      503               ! all other food and drinking places                                                               7223-4, 722514-5
      504               ! automotive repair and maintenance, except car washes                                             81111-2, 811191,811198
      505               ! car washes                                                                                       811192
      506               ! electronic and precision equipment repair and maintenance                                        8112
      507               ! commercial and industrial machinery and equipment repair and maintenance                         8113
      508               ! personal and household goods repair and maintenance                                              8114
      509               ! personal care services                                                                           8121
      510               ! death care services                                                                              8122
      511               ! dry-cleaning and laundry services                                                                8123
      512               ! other personal services                                                                          8129
      513               ! religious organizations                                                                          8131
      514               ! grantmaking, giving, and social advocacy organizations                                           8132-3
      515               ! business and professional associations                                                           81391-2
      516               ! labor and civic organizations                                                                    8134, 81393-9
      517               ! private households                                                                               814
      520               ! other federal government enterprises
      523               ! other state government enterprises
      526               ! other local government enterprises
      527               ! used and secondhand goods
      528               ! scrap
      529               ! rest of the world adjustment
      530               ! noncomparable imports
      531               ! employment and payroll of state govt, non-education
      532               ! employment and payroll of state govt, education
      533               ! employment and payroll of local govt, non-education
      534               ! employment and payroll of local govt, education
      535               ! employment and payroll of federal govt, non-military
      536               ! employment and payroll of federal govt, military
    )
    hlt.(               ! healthcare services
      475               ! offices of physicians                                                                            6211
      476               ! offices of dentists                                                                              6212
      477               ! offices of other health practitioners                                                            6213
      478               ! outpatient care centers                                                                          6214
      479               ! medical and diagnostic laboratories                                                              6215
      480               ! home health care services                                                                        6216
      481               ! other ambulatory health care services                                                            6219
      482               ! hospitals                                                                                        622
    )
  /;


* ------------------------------------------------------------------------------
* define the aggregated households
* ------------------------------------------------------------------------------

* define the aggregated households
set
  h_a                   aggregated households /
    ra                  ! regional representative agent
  /;

* aggregated household mapping
set
  h_map(h_a,h)          aggregated household mapping /
    ra.(                ! regional representative agent
      hhl               ! household income <15k
      hh15              ! household income 15k-30k
      hh30              ! household income 30k-40k
      hh40              ! household income 40k-50k
      hh50              ! household income 50k-70k
      hh70              ! household income 70k-100k
      hh100             ! household income 100k-150k
      hh150             ! household income 150-200k
      hh200             ! household income >200k
    )
  /;
