# basic_example.R
#
# example of how to work with the model and results from R using the SAGE
# utilities. this example:
#    1. runs the baseline model
#    2. runs the sample regulation in examples/sample_abatement_requirement.gms
#    3. presents the change in key variables
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# run the model
# ------------------------------------------------------------------------------

# run the baseline
run.model(output_file="baseline.csv")

# run the policy scenario
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file="examples\\\\sample_abatement_requirement.gms")



# ------------------------------------------------------------------------------
# display the results
# ------------------------------------------------------------------------------

# load the results
bau = read.csv(file.path("output","baseline.csv"),stringsAsFactors=FALSE)
policy = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)

# list of parameters to output results for
vars = c("gdp","cd","leisure")

# calculate the percentage change in the selected variables
ind = bau$parameter %in% vars
diff = aggregate(bau$value[ind],by=list(bau$parameter[ind],bau$time[ind]),sum)
diff$x = (aggregate(policy$value[ind],
                    by=list(policy$parameter[ind],policy$time[ind]),sum)$x/
          diff$x-1)*100

# plot the change in variables
p = ggplot(diff)+
      geom_line(aes(x=Group.2,y=x,linetype=Group.1))+
      labs(y="Change in Variable [%]",
           x="Year")+
      scale_linetype_manual(values=c("solid","dashed","dotted"))+
      guides(linetype=guide_legend(title="Variable"))+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            legend.key.width = unit(1.25,"cm"))

print(p)

# output equivalent variation
ev = policy[policy$parameter=="ev_inf",]
ev = aggregate(ev$value,by=list(ev$household),sum)
writeLines("EV:")
for (i in 1:nrow(ev))
  writeLines(paste0(ev$Group.1[i],"    ",round(ev$x[i],2)))
writeLines(paste0("Total  ",round(sum(ev$x),2)))
