$title basic_example.gms
*
* example of how to work with the model. this example:
*    1. runs the baseline model
*    2. reruns the models with the policy scenario
*    3. presents the change in key variables (windows only)
*
* this example is intended to be run from the top level directory of the model
*
* ------------------------------------------------------------------------------
* run the model
* ------------------------------------------------------------------------------

* use the default aggregation benchmark file
$if not set benchmark_file $set benchmark_file "default_aggregation.gdx"

* use the default parameter file
$if not set parameter_file $set parameter_file "parameters.gms"

* specify OS dependent separator
$if not set sep $set sep "\"

* run the baseline
$onecho>%gams.scrdir%baseline.txt
--gdx_save=1
--output_file=output%sep%baseline.csv
--gdx_results_file=output%sep%baseline.gdx
--benchmark_file=data%sep%%benchmark_file%
--parameter_file=data%sep%%parameter_file%
* set remaining relevant model options to default values for calculation:
--putty_clay=1
--policy_file=0
--gdx_baseline_file=0
--balanced_start_values=0
$offecho
$call 'gams model\sage.gms parmFile="%gams.scrdir%baseline.txt"'

* run the policy scenario
$onecho>%gams.scrdir%policy.txt
--gdx_save=1
--policy_file=examples%sep%sample_abatement_requirement.gms
--gdx_baseline_file=output%sep%baseline.gdx
--output_file=output%sep%policy.csv
--gdx_results_file=output%sep%policy.gdx
--benchmark_file=data%sep%%benchmark_file%
--parameter_file=data%sep%%parameter_file%
* set remaining relevant model options to default values for calculation:
--putty_clay=1
--balanced_start_values=0
$offecho
$call 'gams model\sage.gms parmFile="%gams.scrdir%policy.txt"'

* ------------------------------------------------------------------------------
* display the results (R is much more suited for this task)
* ------------------------------------------------------------------------------

* run csv2gdx for post-solve processing
$call 'csv2gdx output%sep%baseline.csv output=output%sep%baseline_read.gdx ID=baseline Index=(1..6) Values=7 UseHeader=Y FieldSep=Comma'
$call 'csv2gdx output%sep%policy.csv output=output%sep%policy_read.gdx ID=policy Index=(1..6) Values=7 UseHeader=Y FieldSep=Comma'

* read in baseline and policy reports
set
    p	Reported parameter,
    y	Time (in years),
    r	Region,
    s	Sector,
    h	Household,
    sf	Sector from (for two dimensional parameter reports);

parameter
    baseline(p,y,r,s,h,sf)    Baseline values to compare impacts of policy,
    policy(p,y,r,s,h,sf)      Solution values post shock;

$gdxin output%sep%policy_read.gdx
$loaddc p<=policy.dim1 y<=policy.dim2 r<=policy.dim3
$loaddc s<=policy.dim4 h<=policy.dim5 sf<=policy.dim6
$loaddc policy
$gdxin output%sep%baseline_read.gdx
$loaddc baseline

* list of parameters to output results for:
set
    vars(p) /'gdp', 'cd', 'ld', 'leisure' /;

* aggregate chosen parameters to be reported annually (level, pct. ch):
parameter
    report(vars,y,*)    Basic report of model parameter changes;

report(vars,y,'levch') = sum((r,s,h,sf), policy(vars,y,r,s,h,sf)) -
    sum((r,s,h,sf), baseline(vars,y,r,s,h,sf));

report(vars,y,'pctch')$sum((r,s,h,sf), baseline(vars,y,r,s,h,sf)) =
    100 * (sum((r,s,h,sf), policy(vars,y,r,s,h,sf)) /
    sum((r,s,h,sf), baseline(vars,y,r,s,h,sf)) - 1);

* report equivalent variation:
parameter
    ev(*)    Report total and household equivalent variation;

ev(h) = sum((y,r,s,sf), policy('ev_inf',y,r,s,h,sf));
ev('total') = sum(h, ev(h));

* display reports in command line window
file rep / 'output%sep%report.txt' /;
rep.pc = 6;
put rep;
put "Parameter", "Year", "Level Changes", "Pct. Changes" /;
loop((vars,y)$report(vars,y,'levch'), put vars.tl, y.tl, report(vars,y,'levch'), report(vars,y,'pctch') /);
put " " /;
put "Equivalent variation (value)" /;
loop(h$ev(h), put h.tl, ev(h)/);
put "total", ev('total');
putclose;
execute 'type output%sep%report.txt';
