# SAGE

**S**AGE is an **A**pplied **G**eneral **E**quilibrium (SAGE) model of the U.S. economy.

v1.2.0

SAGE is computable general equilibrium model (CGE) of the United States economy developed to aid in the analysis of environmental regulations and policies. It is an intertemporal model with perfect foresight, resolved at the sub-national level. Each of the nine regions in the model, representing the nine census divisions, has five households reflective of national income quintiles. Each region has 23 representative firms, most of which are focused on the manufacturing and energy sectors that are often impacted by environmental policies. Production technologies are represented with nested CES functions, which may include natural resource inputs. Capital for these firms is represented in a partial putty-clay framework to aid in capturing transition dynamics. A single government agent levies taxes on labor earnings, capital earnings, production, and consumption. The United States is treated as a small open economy using the Armington framework governing both domestic and international trade. The baseline is calibrated to the Energy Information Administration’s Annual Energy Outlook. The model is solved using the General Algebraic Modeling System (GAMS) and the PATH solver, with wrappers and data functions written in R. Both GAMS and R need to be added to the system's path before the build stream and modeling environment can be used.

Each directory contains a readme.txt file describing the contents of the directory. For complete documentation of the model, along with instructions for using the model, please see ./documentation/documentation.pdf.

| Directory       | Contents                                                |
|:----------------|:--------------------------------------------------------|
| ./build         | Build stream code and data                              |
| ./data          | Benchmark data nd parameter files for running the model |
| ./documentation | Model documentation, including typeset copy             |
| ./examples      | Examples of running the model                           |
| ./model         | Core SAGE model                                         |
| ./output        | Model output is stored here by default                  |
| ./utilities     | Utilities for running the model and working with output |

## Data Availability

The social accounting matrix for SAGE is based on the state-level IMPLAN data set available from www.IMPLAN.com. This is a proprietary data set and is therefore not included in the publicly available version of SAGE. Given a licensed version of IMPLAN, the SAGE data sets can be build by first following the instructions in ./build/data/implan/implan_data_instructions.txt to add the necessary IMPLAN data files into the SAGE directory structure. Then, run the main build stream function ./build/build_default_data_sets.R. An internet connection is required to run the model's build stream.

An identical set of files, but with the IMPLAN data set included, is being provided to external reviewers to facilitate review of the SAGE model by the U.S. Environmental Protection Agency's (EPA) Science Advisory Board (SAB). The EPA's license with IMPLAN allows us to provide these data under the limitations that it will only be used for the purposes of the review, all copies will be deleted upon competition of the review, and use of the data during the review is consistent with the license agreement (e.g., cannot be shared).
