SAGE Versioning
===================
Based on semantic versioning (http://semver.org/) adapted to a modeling setting.

> Versions are denoted using a triplet of integers: MAJOR.MINOR.PATCH. The basic intent is that MAJOR versions are large-scale upgrades of the model. MINOR versions retain general compatibility with older minor versions, and changes in the PATCH level are typically compatible, forwards and backwards.


Git workflow
============
The following describes the Git workflow that should be followed for SAGE.

The different types of branches used in SAGE are:

* `master`
	* production releases

* `development`
	* AKA integration branch

* Release branches (`release-*`)
	* for finalizing a major/minor release, branched from `development`

* Hotfix branches (`hotfix-*`)
	* for applying patches, branched from master (or support-* for older releases)

* Support branches (`support-*`)
	* for applying patches to old release versions, branched from `master`

* Feature branches (`feature-*`)
	* for developing new features, branched from `development`

Feature branches
----------------
Created for feature development that may require several or more commits to produce a working tree. May make occasional merges from `development` to keep it up to date. If it will be separate for an extended period of time, create a feature-branch-readme.txt in the top level directory dictating the reason for the branch.

**May branch off from**: `development`

	$ git checkout -b feature-newfeature development

**Must merge back into**: `development`

	$ git checkout development
	$ git merge --no-ff feature-newfeature
	$ git branch -d feature-newfeature
	$ git push origin development

Release branches
----------------
Release branches are created when the `development` branch is at a stable point and release specific changes need to be made, such as bumping version numbers. At that point, `development` should be branched and updated as appropriate before ultimately merging it into `master` and tagging the release. There should only be one active release branch at a time. Until the current release is finished, merged into `master`, and deleted, development should take place on `development`. When `development` reaches another state of stability, another release branch should be created.

**May branch off from**: `development`

	$ git checkout -b release-1.2 development
	$ ./bump-version.sh 1.2
	$ git commit -a -m "Bumped version number to 1.2"

Here, bump-version.sh is a fictional shell script that changes some files in the working copy to reflect the new version. (This can of course be a manual change, the point being that some files change).

Bug fixes made on a release branch may be merged back into `development` continuously if needed, though ultimately they will be merged in when the release is finalized.

**Must merge back into**: `development` and `master`

	$ git checkout master
	$ git merge --no-ff release-1.2
	$ git tag 1.2

	$ git checkout development
	$ git merge --no-ff release-1.2

At this point the release branch is safe to delete, since the changes are reflected in `development` and `master`. If the branch is remote, [remove the remote branch](http://github.com/guides/remove-a-remote-branch) also.
	$ git branch -d release-1.2

Hotfix branches
---------------
Patches that needed on the most recent production release are applied to a hotfix branch off `master`.  For older releases, hotfixes branch off of a `support-*` branch (explained later).

**May branch off from**: `master`

	$ git checkout -b hotfix-1.2.1 master
	$ ./bump-version.sh 1.2.1
	$ git commit -a -m "Bumped version number to 1.2.1"

	$ git commit -m "Fixed severe production problem"

**Must merge back into**: `development` and `master`

	$ git checkout master
	$ git merge --no-ff hotfix-1.2.1
	$ git tag 1.2.1

Merge into `development` only if there is no current release branch, otherwise, merge into release branch instead.

	$ git checkout development
	$ git merge --no-ff hotfix-1.2.1

Finally, delete.
	$ git branch -d hotfix-1.2.1

Support branches
----------------

If `master` has moved on a point release (1.0, 1.1, 2.0, etc) and a hotfix must be applied to a older version ( e.g 1.x):

* create a `support-1.x` branch (if none exists) based on the newest 1.x tag in `master`
* create a branch (e.g. `hotfix-1.1.1`), based on `support-1.x`
* fix the bug and merge `hotfix-1.1.1` back into `support-1.x`
* do this for other older major releases as necessary

The support branch effectively becomes a master branch for a past version.
